#define  PI  3.14159
#define  SNUM    50      // サイン波半サイクル分割数
#define  SNUMH  SNUM/2  // ピーク位置
#define  DLY   177      // 200usディレイ用、波形を観測して調整する
#define  ALMLVL  410    // シャント抵抗電圧2V=2A

int  sdata[SNUM];

void  setup() {
  float  fval;
  int  i;

  digitalWrite(12,LOW);
  digitalWrite(11,LOW);
  pinMode(12,OUTPUT);     // ハイサイド1
  pinMode(11,OUTPUT);     // ハイサイド2
  pinMode(8,INPUT);       // 予備入力
  digitalWrite(8,HIGH);   // 予備入力プルアップ

  TCCR1B = (TCCR1B & 0xf8) | 0x01;  // Timer1:CLKIO/1 PWM9,10:31.37KHz
  Serial.begin(9600);  // モニタ用
  for(i=0;i<SNUM;i++) {    // 50分割サイン波データ作成
    fval = i*PI/SNUM;
    sdata[i]=(int)(sin(fval)*255);
    Serial.println(sdata[i],DEC);  // データ確認
  }
}

void  loop() {
  int  i,j,adata,sl;

  for(j=0,sl=0;;) {
    sl += 4;            // ソフトスタート変数計算
    if(sl>127) sl=128;  // 32サイクルで飽和
    if(j==0) {          // ブザー処理(鳴らしていないときはpin7へ出力)
      noTone(6);
      tone(7,4000);
    } 
    else {
      noTone(7);
      tone(6,4000);
    }
    // 正サイクル開始
    analogWrite(10,0);       // ローサイド0
    analogWrite(9,0);
    digitalWrite(12,LOW);    // ハイサイド1 off
    delayMicroseconds(DLY);
    digitalWrite(11,HIGH);   // ハイサイド2 on
    for(i=1;i<SNUM;i++) {
      analogWrite(9,(sdata[i]*sl)>>7);  // サイン波データ出力
      if(i==SNUMH) {            // 波形のピーク?
        adata = analogRead(1);  // 過電流監視
        if(adata>ALMLVL) alarm();
        delayMicroseconds(DLY-110);	// A/D変換にも110uSかかる
      } 
      else {
        delayMicroseconds(DLY);
      }
    }
    // 負サイクル開始
    analogWrite(10,0);       // ローサイド0
    analogWrite(9,0);
    digitalWrite(11,LOW);    // ハイサイド2 off
    delayMicroseconds(DLY);
    digitalWrite(12,HIGH);    // ハイサイド1 on
    for(i=1;i<SNUM;i++) {
      analogWrite(10,(sdata[i]*sl)>>7);  // サイン波データ出力
      if(i==SNUMH) {            // 波形のピーク?
        adata = analogRead(1);  // 過電流監視
        if(adata>ALMLVL) alarm();
        delayMicroseconds(DLY-110);	// A/D変換にも110uSかかる
      } 
      else {
        delayMicroseconds(DLY);
      }
    }
    if(++j>50) j=0;
  }
}

// 過電流検出
void  alarm(void)
{
  digitalWrite(11,LOW);
  digitalWrite(12,LOW);
  analogWrite(10,0);
  analogWrite(9,0);
  noTone(6);
  for(;;) {		// 電源オフまで、ブザー鳴動
    tone(7,4000,500);
    delay(500);
    delay(500);
  }
}


